/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.jmx.topology;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Container.NodeType;
import org.ow2.petals.admin.topology.Container.PortType;
import org.ow2.petals.admin.topology.Domain;
import org.ow2.petals.admin.topology.Subdomain;
import org.ow2.petals.jmx.commons.PetalsAdminServiceConstants;
import org.ow2.petals.jmx.commons.PetalsAdminServiceConstants.Topology;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;

/**
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class TopologyUtils {

    private static final Map<String, PortType> portTypes;
    
    static {
        HashMap<String, Container.PortType> tempPortTypes = new HashMap<String, Container.PortType>();
        tempPortTypes.put(PetalsAdminServiceConstants.Container.CONF_WEBSERVICE_HTTP_PORT, PortType.HTTP_WEBSERVICE);
        tempPortTypes.put(PetalsAdminServiceConstants.Container.CONF_JMX_RMI_PORT, PortType.JMX);
        tempPortTypes.put(PetalsAdminServiceConstants.Container.CONF_TRANSPORT_TCP_PORT, PortType.TCP_TRANSPORT);
        tempPortTypes.put(PetalsAdminServiceConstants.Container.CONF_REGISTRY_PORT, PortType.REGISTRY);
        portTypes = Collections.unmodifiableMap(tempPortTypes);
    }   
    
    public static final Domain createListOfDomains(Set<Map<String,String>> topology) {
        Domain domain = null;
        List<Subdomain> subdomains = new ArrayList<Subdomain>();
        HashMap <String, List<Container>> containers = new HashMap<String, List<Container>>();
        
        for(Map<String,String> map : topology) {
            String topologyType = map.get(Topology.TOPOLOGY_TYPE);
            
            if(topologyType.equals(PetalsAdminServiceConstants.Domain.TOPOLOGY_TYPE)) {
                String name = map.get(PetalsAdminServiceConstants.Domain.CONF_DOMAIN_NAME);
                domain = new Domain(name);
            } else if(topologyType.equals(PetalsAdminServiceConstants.SubDomain.TOPOLOGY_TYPE)) {
                String name  = map.get(PetalsAdminServiceConstants.SubDomain.CONF_SUBDOMAIN_NAME);
                Subdomain subdomain = new Subdomain(name);
                subdomains.add(subdomain);
            } else if(topologyType.equals(PetalsAdminServiceConstants.Container.TOPOLOGY_TYPE)) {
                String subdomain =  map.get(PetalsAdminServiceConstants.Container.CONF_SUBDOMAIN_NAME);
                
                List<Container> containersForThisSubDomain;
                if(containers.containsKey(subdomain)) {
                    containersForThisSubDomain = containers.get(subdomain);
                } else {
                    containersForThisSubDomain = new ArrayList<Container>();
                    containers.put(subdomain, containersForThisSubDomain);
                }
                
                String name  = map.get(PetalsAdminServiceConstants.Container.CONF_CONTAINER_NAME);
                NodeType nodeType = NodeType.valueOf(map.get(PetalsAdminServiceConstants.Container.CONF_NODE_TYPE));
                String host  = map.get(PetalsAdminServiceConstants.Container.CONF_HOST);
                final String jmxUsername = map.get(PetalsAdminServiceConstants.Container.CONF_USER);
                final String jmxPassword = map.get(PetalsAdminServiceConstants.Container.CONF_PWD);
                
                HashMap<PortType, Integer> ports = new HashMap<PortType, Integer>();
                for(String portType : portTypes.keySet()) {
                    if(map.containsKey(portType)) {
                        int portValue =Integer.valueOf(map.get(portType));
                        ports.put(portTypes.get(portType), portValue);
                    }
                }
                
                Container container = new Container(name, nodeType, host, ports, jmxUsername,
                        jmxPassword);
                containersForThisSubDomain.add(container);
            } else {
                String errorMessage = String.format("Unrecognized topology type: '%s'", topologyType);
                throw new UncheckedException(errorMessage);
            }
        }
        
        if(domain == null) {
            String errorMessage = "Error in the topology map: no domain";
            throw new UncheckedException(errorMessage);
        }
        
        for(Subdomain subdomain : subdomains) {
            subdomain.addContainers(containers.get(subdomain.getName()));
        }        
        domain.addSubdomain(subdomains);
        
        return domain;
    }
    
}
